﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Master
{
    class MatrixHelper
    {
        /// <summary>
        /// Rozpuleni Vektoru
        /// </summary>
        /// <typeparam name="T">datovy typ</typeparam>
        /// <param name="e">vektor pro prepuleni</param>
        /// <param name="secondHalf">druha polovina vektoru</param>
        /// <returns></returns>
        public static T[] HalfOfVector<T>(T[] e, bool secondHalf = false)
        {
            var H = new T[(int)Math.Floor(((double)(e.Length / 2)))];

            for (int i = 0; i < H.Length; i++)
            {
                H[i] = e[secondHalf ? (i + H.Length) : i];
            }

            return H;
        }

        /// <summary>
        /// Rozctvrceni Matice
        /// </summary>
        /// <typeparam name="T">datovy typ</typeparam>
        /// <param name="source">matice</param>
        /// <param name="first">prvni ctvrtina</param>
        /// <param name="second">druha ctvrtina</param>
        /// <param name="third">treti ctvrtina</param>
        /// <param name="fourth">ctvrta ctvrtina</param>
        public static void QuartersOfMatrix<T>(T[,] source, out T[,] first, out T[,] second, out T[,] third, out T[,] fourth)
        {
            var C = (int)Math.Floor((double)(source.GetLength(1) / 2)); // 4
            var R = (int)Math.Floor((double)(source.GetLength(0) / 2)); // 5

            first = new T[R, C];
            second = new T[R, C];
            third = new T[R, C];
            fourth = new T[R, C];

            for (int x = 0; x < (R * 2); x++)
            {
                for (int y = 0; y < (C * 2); y++)
                {
                    if (x < R)
                    {
                        if (y < C) first[x, y] = source[x, y];
                        else second[x, y - C] = source[x, y];
                    }
                    else
                    {
                        if (y < C) third[x - R, y] = source[x, y];
                        else fourth[x - R, y - C] = source[x, y];
                    }
                }
            }
        }



        /// <summary>
        /// Vynasobeni Matice Vektorem a soucet hodnot
        /// </summary>
        /// <param name="M">matice</param>
        /// <param name="V">vektor</param>
        /// <returns>vysledny vektor</returns>
        public static double[] MultipleByVectorAndSumIt(double[,] M, double[] V)
        {
            var N = new double[M.GetLength(0)];

            for (int r = 0; r < N.Length; r++)
            {
                double v = 0;
                for (int c = 0; c < M.GetLength(1); c++)
                {
                    var t = M[r, c];

                    try
                    {
                        t = t * V[c];
                    }
                    catch
                    {
                        Console.WriteLine("CHYBA: v prepoctu pro DOP2");
                    }

                    v = v + t;
                }

                N[r] = v;
            }

            return N;
        }

        /// <summary>
        /// Odecteni dvou vekotru od sebe
        /// </summary>
        /// <param name="V1">vektor 1</param>
        /// <param name="V2">vektor 2</param>
        /// <returns>vysledny vektor</returns>
        public static double[] SubtractionTwoVectors(double[] V1, double[] V2)
        {
            var N = new double[V1.Length];

                for (int c = 0; c < V1.Length; c++)
                {
                    try
                    {
                        N[c] = V1[c] - V2[c];
                    }
                    catch
                    {
                        Console.WriteLine("CHYBA: v prepoctu pro DOP2");
                    }
                }

            return N;
        }
    }
}
