﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimplexMethod
{
    /// <summary>
    /// Trida resice Simplexove metody
    /// </summary>
    class SimplexMethodSolver
    {
        Simplex simplex;
        private double[,] tableau; // Simplex Tableau - matice + jednotkova matice
        private int numberOfConstraints; // pocet omezujicich (ne)rovnic
        private int numberOfOriginalVariables; // pocet promennych ucelove fce Z
        private SimplexType simplexType; // typ Simplexovy Metody -> MIMIMAZE/MAXIMAZE

        public SimplexMethodSolver(double[,] constraintLeftSide, double[] constraintRightSide, ConstraintOperator constraintOperator, double[] objectiveFunction, SimplexType simplexType)
        {
            numberOfConstraints = constraintRightSide.Length; // pocet vysledku (ne)rovnic
            numberOfOriginalVariables = objectiveFunction.Length; // delka ucelova fce Z

            tableau = new double[numberOfConstraints + 1, numberOfOriginalVariables + numberOfConstraints + 1]; // Simplex Tableau + jednotkova matice - rozmery: [radky, sloupce]

            // naplneni matice omezujicimi (ne)rovnicemi
            for (int i = 0; i < numberOfConstraints; i++)
            {
                for (int j = 0; j < numberOfOriginalVariables; j++)
                {
                    tableau[i, j] = constraintLeftSide[i, j]; // leve strany (ne)rovnic
                }
            }

            for (int i = 0; i < numberOfConstraints; i++)
                tableau[i, numberOfConstraints + numberOfOriginalVariables] = constraintRightSide[i]; // prave strany (vysledne hodnoty) (ne)rovnic

            // pridani jednotkove matice - takzvanych: "slack variables" s
            for (int i = 0; i < numberOfConstraints; i++)
            {
                int slack = 0;
                switch (constraintOperator)
                {
                    case ConstraintOperator.GREATHER_THAN:
                        slack = -1; // pro nerovnici vetsi_rovno >= 
                        break;
                    case ConstraintOperator.LESS_THAN:
                        slack = 1; // pro nerovnici mensi_rovno <=
                        break;
                    case ConstraintOperator.EQUAL:
                        slack = 0; // pro rovnici =
                        break;
                    default:
                        break;
                }
                tableau[i, numberOfOriginalVariables + i] = slack;
            }

            // pridani ucelove funkce do matice
            for (int j = 0; j < numberOfOriginalVariables; j++)
            {
                tableau[numberOfConstraints, j] = objectiveFunction[j];
            }
               
            simplex = new Simplex(tableau, numberOfConstraints, numberOfOriginalVariables, simplexType);
        }


        public void solve()
        {
            simplex.solve();
        }

        public double[,] getTableaux()
        {
            return tableau;
        }

        public double[] getSolutionVector()
        {
            return simplex.solutionVector();
        }

        public double getValue()
        {
            return simplex.value();
        }

        public int getNumberOfConstraint()
        {
            return numberOfConstraints;
        }

        public int getNumberOfOriginalVariable()
        {
            return numberOfOriginalVariables;
        }
    }
}

