﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimplexMethod
{
    public static class TxtFileIO
    {
        public enum TaskType
        {
            MIN = 0,
            MAX
        }

        private const char SeparatorGroup = '|';
        private const char SeparatorItem = ';';

        /// <summary>
        /// Zapis zadani pro Simplexovou metodu do .txt souboru
        /// </summary>
        /// <param name="file_name">nazev souboru + .txt</param>
        /// <param name="Zx">ucelova fce</param>
        /// <param name="A">matice leve strany omezeni</param>
        /// <param name="b">hodnoty prave strany omezeni</param>
        public static void Save(string file_name, double[] Zx, double[,] A, double[] b)
        {
            var data = string.Empty;

            for (int i = 0; i < Zx.Length; i++)
            {
                if (i > 0) data += SeparatorItem;
                data += Zx[i].ToString();
            }

            data += SeparatorGroup;
            data += A.GetLength(1).ToString();
            for (int y = 0; y < A.GetLength(0); y++)
            {
                for (int x = 0; x < A.GetLength(1); x++)
                {
                    data += SeparatorItem;
                    data += A[y, x];
                }
            }

            data += SeparatorGroup;

            for (int i = 0; i < b.Length; i++)
            {
                if (i > 0) data += SeparatorItem;
                data += b[i].ToString();
            }

            File.WriteAllText(file_name, data);
        }

        /// <summary>
        /// Precteni zadani pro Simplexovou metodu z .txt souboru
        /// </summary>
        /// <param name="file_name">nazev souboru + .txt</param>
        /// <param name="Zx">ucelova fce</param>
        /// <param name="A">matice leve strany omezeni</param>
        /// <param name="b">hodnoty prave strany omezeni</param>
        public static void Load(string file_name, out double[] Zx, out double[,] A, out double[] b)
        {
            var data = File.ReadAllText(file_name);

            if (string.IsNullOrEmpty(data))
            {
                Zx = null;
                A = null;
                b = null;
            }

            var groups = data.Split(SeparatorGroup);

            var ZxGroup = groups[0].Split(SeparatorItem);
            var AGroup = groups[1].Split(SeparatorItem);
            var bGroup = groups[2].Split(SeparatorItem);

            Zx = new double[ZxGroup.Length];
            for (int i = 0; i < Zx.Length; i++)
            {
                Zx[i] = Convert.ToDouble(ZxGroup[i]);
            }

            var A_L1 = Convert.ToInt32(AGroup[0]);
            var A_L0 = AGroup.Length / A_L1;
            A = new double[A_L0, A_L1];
            for (int x = 0; x < A_L0; x++)
            {
                for (int y = 0; y < A_L1; y++)
                {
                    A[x, y] = Convert.ToDouble(AGroup[((x * A_L1) + y)+ 1]);
                }
            }

            b = new double[bGroup.Length];
            for (int i = 0; i < b.Length; i++)
            {
                b[i] = Convert.ToDouble(bGroup[i]);
            }
        }

        /// <summary>
        /// Zapis vysledku Simlexove metody do .txt souboru
        /// </summary>
        /// <param name="file_name">nazev souboru + .txt</param>
        /// <param name="X"></param>
        /// <param name="Z"></param>
        public static void Save(string file_name, double[] X, double Z)
        {
            var data = string.Empty;

            for (int i = 0; i < X.Length; i++)
            {
                if (i > 0) data += SeparatorItem;
                data += X[i].ToString();
            }
            data += SeparatorItem;
            data += Z.ToString();

            File.WriteAllText(file_name, data);
        }

        /// <summary>
        /// Precteni vysledku Simlexove metody z .txt souboru
        /// </summary>
        /// <param name="file_name">nazev souboru + .txt</param>
        /// <param name="X"></param>
        /// <param name="Z"></param>
        public static void Load(string file_name, out double[] X, out double Z)
        {
            var data = File.ReadAllText(file_name);

            if (string.IsNullOrEmpty(data))
            {
                X = null;
                Z = 0;
            }

            var items = data.Split(SeparatorItem);

            X = new double[items.Length - 1];
            Z = Convert.ToDouble(items[items.Length - 1]);
            for (int i = 0; i < (items.Length - 1); i++)
            {
                X[i] = Convert.ToDouble(items[i]);
            }
        }

        /// <summary>
        /// Precteni zadani z .txt souboru
        /// </summary>
        /// <param name="file_name">nazev souboru + .txt</param>
        /// <param name="Zx">ucelova fce</param>
        /// <param name="A">matice leve strany omezeni</param>
        /// <param name="b">hodnoty prave strany omezeni</param>
        /// <param name="t">typ ulohy MAX/MIN</param>
        public static void ReadInput(string file_name, out double[] Zx, out double[,] A, out double[] b, out TaskType t)
        {
            // [radek,sloupec]

            var data = File.ReadAllText(file_name);

            if (string.IsNullOrEmpty(data))
            {
                if (string.IsNullOrEmpty(data))
                {
                    Zx = null;
                    A = null;
                    b = null;
                }
            }

            var groups = data.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);

            var ZxGroup = groups[0].Split(SeparatorItem);
            var bGroup = groups[groups.Length - 2].Split(SeparatorItem);

            var A_V0 = groups[1].Split(SeparatorItem);
            var A_L0 = groups.Length - 3;
            var A_L1 = A_V0.Length;
            A = new double[A_L0, A_L1];
            for (int x = 0; x < A_L0; x++)
            {
                var row = groups[x + 1].Split(SeparatorItem);
                for (int y = 0; y < A_L1; y++)
                {
                    A[x, y] = Convert.ToDouble(row[y]);
                }
            }

            Zx = new double[ZxGroup.Length];
            for (int i = 0; i < Zx.Length; i++)
            {
                Zx[i] = Convert.ToDouble(ZxGroup[i]);
            }

            b = new double[bGroup.Length];
            for (int i = 0; i < b.Length; i++)
            {
                b[i] = Convert.ToDouble(bGroup[i]);
            }

            t = (groups[groups.Length - 1].ToLower() == "min") ? TaskType.MIN : TaskType.MAX;
        }
    }
}
